<?php
/**
 * Soap Client using Microsoft's NTLM Authentication
 * 
 * Copyright (c) 2008 Invest-In-France Agency http://www.invest-in-france.org
 * 
 * Author : Thomas Rabaix
 * 
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * @link http://rabaix.net/en/articles/2008/03/13/using-soap-php-with-ntlm-authentication
 * @author Thomas Rabaix
 */

/**
 * Soap Client using Microsoft's NTLM Authentication
 * 
 * @link http://rabaix.net/en/articles/2008/03/13/using-soap-php-with-ntlm-authentication
 * @author Thomas Rabaix
 */
class NTLMSoapClient extends SoapClient {
    /**
    * Username for authentication on the exchnage server
         * 
    * @var string
         */
    protected $user;
    
    /**
     * Password for authentication on the exchnage server
     * 
     * @var string
     */
    protected $password;
    
    /**
     * Server
     *
     * @var string
     *
     */    
    protected $server;
    
    /**
     * Whether or not to validate ssl certificates
     * 
     * @var boolean
     */
    protected $validate = false;
    
    protected $ch = null;
    
    public function __construct($wsdl, $options, $server, $user, $password) {
        $this->server = $server;
        $this->user = $user;
        $this->password = $password;
        parent::__construct($wsdl, $options);
    }
    
    public function __destruct() {
        $this->closeSocket();
    }
    
    private function closeSocket() {
        if($this->ch) {
            curl_close($this->ch);
            $this->ch = null;
        }
    }
    
    /**
     * Performs a SOAP request
     * 
     * @link http://php.net/manual/en/function.soap-soapclient-dorequest.php
     * 
     * @param string $request the xml soap request
     * @param string $location the url to request
     * @param string $action the soap action.
     * @param integer $version the soap version
     * @param integer $one_way
     * @return string the xml soap response.
     */
    public function __doRequest($request, $location, $action, $version, $one_way = 0) {
        $headers = array(
            'Method: POST',
            'Connection: Keep-Alive',
            'User-Agent: PHP-SOAP-CURL',
            'Content-Type: text/xml; charset=utf-8',
            'SOAPAction: "'.$action.'"',
        ); // end $headers
        if(!$this->ch) {
            $this->ch = curl_init($location);
        }
        
        curl_setopt($this->ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($this->ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($this->ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($this->ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($this->ch, CURLOPT_POST, true );
        curl_setopt($this->ch, CURLOPT_POSTFIELDS, $request);
        curl_setopt($this->ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($this->ch, CURLOPT_USERPWD, $this->user.':'.$this->password);
        curl_setopt($this->ch, CURLOPT_CONNECTTIMEOUT, 10);
        curl_setopt($this->ch, CURLOPT_TIMEOUT, 65);
        curl_setopt($this->ch, CURLOPT_HTTPAUTH, CURLAUTH_NTLM | CURLAUTH_BASIC | CURLAUTH_DIGEST);
        
        $response = curl_exec($this->ch);
        if($response === false) {
            $error = curl_error($this->ch)." (".curl_errno($this->ch).")";
            $this->closeSocket();
            throw new Exception($error);
        }
        elseif(($httpStatus = curl_getinfo($this->ch, CURLINFO_HTTP_CODE)) !== 200) {
            $this->closeSocket();
            $msg = "Curl HTTP Status $httpStatus";
            if($response != "") {
                $msg .= " ($response)";
            }
            throw new Exception($msg);
        }
        return $response;
    } // end function __doRequest()
} // end class NTLMSoapClient
